@@ -12,7 +12,7 @@
 #include "retro-input.h"
 #include "retro-input-descriptor.h"
 #include "retro-module.h"
-#include "retro-options.h"
+#include "retro-option-private.h"
 #include "retro-pixel-format.h"
 #include "retro-rotation.h"
 
@@ -51,7 +51,10 @@ struct _RetroCore
   GtkWidget *keyboard_widget;
   gulong key_press_event_id;
   gulong key_release_event_id;
-  RetroOptions *options;
+  GHashTable *options;
+  gboolean variable_updated;
+  guint runahead;
+  gssize run_remaining;
 };
 
 void retro_core_push_cb_data (RetroCore *self);
@@ -63,6 +66,8 @@ void retro_core_set_support_no_game (RetroCore *self,
 gchar *retro_core_get_name (RetroCore *self);
 void retro_core_set_system_av_info (RetroCore         *self,
                                     RetroSystemAvInfo *system_av_info);
+void retro_core_set_geometry (RetroCore         *self,
+                              RetroGameGeometry *geometry);
 void retro_core_poll_controllers (RetroCore *self);
 gint16 retro_core_get_controller_input_state (RetroCore  *self,
                                               uint        port,
@@ -74,6 +79,10 @@ void retro_core_set_controller_port_device (RetroCore           *self,
 void retro_core_set_controller_descriptors (RetroCore            *self,
                                             RetroInputDescriptor *input_descriptors,
                                             gsize                 length);
+gboolean retro_core_is_running_ahead (RetroCore *self);
+void retro_core_insert_variable (RetroCore           *self,
+                                 const RetroVariable *variable);
+gboolean retro_core_get_variable_update (RetroCore *self);
 
 G_END_DECLS
 
