@@ -5,6 +5,7 @@
 #include <string.h>
 #include "retro-controller-iterator-private.h"
 #include "retro-keyboard-key.h"
+#include "retro-option-iterator-private.h"
 #include "retro-pixdata.h"
 
 #define RETRO_CORE_ERROR (retro_core_error_quark ())
@@ -31,6 +32,7 @@ enum {
   PROP_GAME_LOADED,
   PROP_SUPPORT_NO_GAME,
   PROP_FRAMES_PER_SECOND,
+  PROP_RUNAHEAD,
   N_PROPS,
 };
 
@@ -42,6 +44,7 @@ enum {
   SIG_LOG_SIGNAL,
   SIG_SHUTDOWN_SIGNAL,
   SIG_MESSAGE_SIGNAL,
+  SIG_OPTIONS_SET_SIGNAL,
   N_SIGNALS,
 };
 
@@ -151,14 +154,14 @@ retro_core_finalize (GObject *object)
     if (self->default_controllers[i] != NULL)
       g_object_unref (self->default_controllers[i]);
   g_hash_table_unref (self->controllers);
-  g_object_unref (self->options);
+  g_hash_table_unref (self->options);
 
   g_free (self->filename);
   g_free (self->system_directory);
   g_free (self->libretro_path);
   g_free (self->content_directory);
   g_free (self->save_directory);
-  g_object_unref (self->keyboard_widget);
+  g_clear_object (&self->keyboard_widget);
 
   G_OBJECT_CLASS (retro_core_parent_class)->finalize (object);
 }
@@ -191,6 +194,10 @@ retro_core_get_property (GObject    *object,
   case PROP_FRAMES_PER_SECOND:
     g_value_set_double (value, retro_core_get_frames_per_second (self));
 
+    break;
+  case PROP_RUNAHEAD:
+    g_value_set_uint (value, retro_core_get_runahead (self));
+
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -235,6 +242,10 @@ retro_core_set_property (GObject      *object,
   case PROP_SUPPORT_NO_GAME:
     retro_core_set_support_no_game (self, g_value_get_boolean (value));
 
+    break;
+  case PROP_RUNAHEAD:
+    retro_core_set_runahead (self, g_value_get_uint (value));
+
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -392,6 +403,23 @@ retro_core_class_init (RetroCoreClass *klass)
                          G_PARAM_STATIC_NICK |
                          G_PARAM_STATIC_BLURB);
 
+  /**
+   * RetroCore:runahead:
+   *
+   * The number of frame to run ahead of time.
+   */
+  properties[PROP_RUNAHEAD] =
+    g_param_spec_uint ("runahead",
+                       "Runahead",
+                       "The number of frame to run ahead of time",
+                       0,
+                       G_MAXUINT,
+                       0,
+                       G_PARAM_READWRITE |
+                       G_PARAM_STATIC_NAME |
+                       G_PARAM_STATIC_NICK |
+                       G_PARAM_STATIC_BLURB);
+
   g_object_class_install_properties (G_OBJECT_CLASS (klass), N_PROPS, properties);
 
   /**
@@ -490,11 +518,27 @@ retro_core_class_init (RetroCoreClass *klass)
                   2,
                   G_TYPE_STRING,
                   G_TYPE_UINT);
+
+  /**
+   * RetroCore::options-set:
+   * @self: the #RetroCore
+   *
+   * The ::options-set signal is emitted when the core sets the
+   * options during boot.
+   */
+  signals[SIG_OPTIONS_SET_SIGNAL] =
+    g_signal_new ("options-set", RETRO_TYPE_CORE, G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  NULL,
+                  G_TYPE_NONE,
+                  0);
 }
 
 static void
 retro_core_init (RetroCore *self)
 {
+  self->options = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                         g_free, g_object_unref);
 }
 
 static void
@@ -569,6 +613,29 @@ retro_core_get_needs_full_path (RetroCore *self)
   return system_info.need_fullpath;
 }
 
+void
+retro_core_set_system_av_info (RetroCore         *self,
+                               RetroSystemAvInfo *system_av_info)
+{
+  if (self->frames_per_second != system_av_info->timing.fps) {
+    self->frames_per_second = system_av_info->timing.fps;
+    g_object_notify (G_OBJECT (self), "frames-per-second");
+  }
+  retro_core_set_geometry (self, &system_av_info->geometry);
+  self->sample_rate = system_av_info->timing.sample_rate;
+}
+
+void
+retro_core_set_geometry (RetroCore         *self,
+                         RetroGameGeometry *geometry)
+{
+  if (geometry->aspect_ratio > 0.f)
+    self->aspect_ratio = geometry->aspect_ratio;
+  else
+    self->aspect_ratio = (float) geometry->base_width /
+                         (float) geometry->base_height;
+}
+
 /**
  * retro_core_get_name:
  * @self: a #RetroCore
@@ -922,7 +989,7 @@ retro_core_load_medias (RetroCore *self,
   GError *tmp_error = NULL;
 
   g_return_if_fail (RETRO_IS_CORE (self));
-  length = g_strv_length (self->media_uris);
+  length = self->media_uris == NULL ? 0 : g_strv_length (self->media_uris);
 
   if (length == 0) {
     retro_core_prepare (self);
@@ -990,6 +1057,60 @@ on_key_event (GtkWidget   *widget,
   return retro_core_key_event (RETRO_CORE (self), event);
 }
 
+static void
+on_option_value_changed (RetroOption *option,
+                         RetroCore   *self)
+{
+  g_return_if_fail (RETRO_IS_CORE (self));
+
+  self->variable_updated = TRUE;
+}
+
+void
+retro_core_insert_variable (RetroCore           *self,
+                            const RetroVariable *variable)
+{
+  RetroOption *option;
+  const gchar *key;
+  GError *tmp_error = NULL;
+
+  g_return_if_fail (RETRO_IS_CORE (self));
+  g_return_if_fail (variable != NULL);
+
+  option = retro_option_new (variable, &tmp_error);
+  if (G_UNLIKELY (tmp_error != NULL)) {
+    g_debug ("%s", tmp_error->message);
+    g_clear_error (&tmp_error);
+
+    return;
+  }
+
+  key = retro_option_get_key (option);
+
+  g_hash_table_insert (self->options, g_strdup (key), option);
+
+  g_signal_connect_object (option,
+                           "value-changed",
+                           G_CALLBACK (on_option_value_changed),
+                           self,
+                           0);
+
+  self->variable_updated = TRUE;
+}
+
+gboolean
+retro_core_get_variable_update (RetroCore *self)
+{
+  g_return_val_if_fail (RETRO_IS_CORE (self), FALSE);
+
+  if (!self->variable_updated)
+    return FALSE;
+
+  self->variable_updated = FALSE;
+
+  return TRUE;
+}
+
 /* Public */
 
 /**
@@ -1396,13 +1517,109 @@ void
 retro_core_run (RetroCore *self)
 {
   RetroRun run;
+  RetroSerializeSize serialize_size = NULL;
+  RetroSerialize serialize = NULL;
+  RetroUnserialize unserialize = NULL;
+  guint8 *data;
+  gsize size;
+  gsize new_size;
+  gboolean success;
 
   g_return_if_fail (RETRO_IS_CORE (self));
 
-  retro_core_push_cb_data (self);
   run = retro_module_get_run (self->module);
+
+  if (self->runahead == 0) {
+    self->run_remaining = 0;
+    retro_core_push_cb_data (self);
+    run ();
+    retro_core_pop_cb_data ();
+
+    return;
+  }
+
+  serialize_size = retro_module_get_serialize_size (self->module);
+
+  retro_core_push_cb_data (self);
+  size = serialize_size ();
+  retro_core_pop_cb_data ();
+
+  if (size == 0) {
+    self->run_remaining = 0;
+    retro_core_push_cb_data (self);
+    run ();
+    retro_core_pop_cb_data ();
+
+    g_critical ("Couldn't run ahead: serialization not supported.");
+
+    return;
+  }
+
+  self->run_remaining = self->runahead;
+
+  retro_core_push_cb_data (self);
   run ();
   retro_core_pop_cb_data ();
+
+  self->run_remaining--;
+
+  new_size = serialize_size ();
+
+  if (size > new_size) {
+    g_critical ("Couldn't run ahead: unexpected serialization size %"
+                G_GSIZE_FORMAT", expected %"G_GSIZE_FORMAT" or less.",
+                new_size, size);
+
+    return;
+  }
+
+  size = new_size;
+  data = g_new0 (guint8, size);
+
+  serialize = retro_module_get_serialize (self->module);
+  retro_core_push_cb_data (self);
+  success = serialize (data, size);
+  retro_core_pop_cb_data ();
+
+  if (!success) {
+    g_critical ("Couldn't run ahead: serialization unexpectedly failed.");
+
+    g_free (data);
+
+    return;
+  }
+
+  retro_core_push_cb_data (self);
+  for (; self->run_remaining >= 0; self->run_remaining--)
+    run ();
+  retro_core_pop_cb_data ();
+
+  new_size = serialize_size ();
+
+  if (size > new_size) {
+    g_critical ("Couldn't run ahead: unexpected deserialization size %"
+                G_GSIZE_FORMAT", expected %"G_GSIZE_FORMAT" or less.",
+                new_size, size);
+
+    g_free (data);
+
+    return;
+  }
+
+  unserialize = retro_module_get_unserialize (self->module);
+  retro_core_push_cb_data (self);
+  success = unserialize ((guint8 *) data, size);
+  retro_core_pop_cb_data ();
+
+  if (!success) {
+    g_critical ("Couldn't run ahead: deserialization unexpectedly failed.");
+
+    g_free (data);
+
+    return;
+  }
+
+  g_free (data);
 }
 
 /**
@@ -1876,6 +2093,85 @@ retro_core_iterate_controllers (RetroCore *self)
   return retro_controller_iterator_new (self->controllers);
 }
 
+guint
+retro_core_get_runahead (RetroCore *self)
+{
+  g_return_val_if_fail (RETRO_IS_CORE (self), 0);
+
+  return self->runahead;
+}
+
+void
+retro_core_set_runahead (RetroCore *self,
+                         guint      runahead)
+{
+  g_return_if_fail (RETRO_IS_CORE (self));
+
+  self->runahead = runahead;
+}
+
+gboolean
+retro_core_is_running_ahead (RetroCore *self)
+{
+  return self->run_remaining > 0;
+}
+
+/**
+ * retro_core_has_option:
+ * @self: a #RetroCore
+ * @key: the key of the option
+ *
+ * Gets whether the core has an option for the given key.
+ *
+ * Returns: whether the core has an option for the given key
+ */
+gboolean
+retro_core_has_option (RetroCore   *self,
+                       const gchar *key)
+{
+  g_return_val_if_fail (RETRO_IS_CORE (self), FALSE);
+  g_return_val_if_fail (key != NULL, FALSE);
+
+  return g_hash_table_contains (self->options, key);
+}
+
+/**
+ * retro_core_get_option:
+ * @self: a #RetroCore
+ * @key: the key of the option
+ *
+ * Gets the option for the given key.
+ *
+ * Returns: (transfer none): the option
+ */
+RetroOption *
+retro_core_get_option (RetroCore    *self,
+                       const gchar  *key)
+{
+  g_return_val_if_fail (RETRO_IS_CORE (self), NULL);
+  g_return_val_if_fail (key != NULL, NULL);
+
+  return RETRO_OPTION (g_hash_table_lookup (self->options, key));
+}
+
+/**
+ * retro_core_iterate_options:
+ * @self: a #RetroCore
+ *
+ * Creates a new #RetroOptionIterator which can be used to iterate through the
+ * options of @self.
+ *
+ * Returns: (transfer full): a new #RetroOptionIterator
+ */
+RetroOptionIterator *
+retro_core_iterate_options (RetroCore *self)
+{
+
+  g_return_val_if_fail (RETRO_IS_CORE (self), NULL);
+
+  return retro_option_iterator_new (self->options);
+}
+
 /**
  * retro_core_new:
  * @filename: the filename of a Libretro core
@@ -1911,7 +2207,6 @@ retro_core_new (const gchar *filename)
   retro_core_set_callbacks (self);
   self->controllers = g_hash_table_new_full (g_int_hash, g_int_equal,
                                              g_free, g_object_unref);
-  self->options = retro_options_new ();
 
   return self;
 }
