@@ -88,6 +88,7 @@ retro_gl_display_get_video_box (RetroGLDisplay *self,
   gdouble h;
   gdouble display_ratio;
   gdouble allocated_ratio;
+  gint scale;
 
   g_return_if_fail (RETRO_IS_GL_DISPLAY (self));
   g_return_if_fail (width != NULL);
@@ -95,8 +96,10 @@ retro_gl_display_get_video_box (RetroGLDisplay *self,
   g_return_if_fail (x != NULL);
   g_return_if_fail (y != NULL);
 
-  w = (gdouble) gtk_widget_get_allocated_width (GTK_WIDGET (self));
-  h = (gdouble) gtk_widget_get_allocated_height (GTK_WIDGET (self));
+  scale = gtk_widget_get_scale_factor (GTK_WIDGET (self));
+
+  w = (gdouble) gtk_widget_get_allocated_width (GTK_WIDGET (self)) * scale;
+  h = (gdouble) gtk_widget_get_allocated_height (GTK_WIDGET (self)) * scale;
 
   // Set the size of the display.
   display_ratio = (gdouble) self->aspect_ratio;
@@ -122,6 +125,8 @@ retro_gl_display_load_texture (RetroGLDisplay *self,
                                gint           *texture_width,
                                gint           *texture_height)
 {
+  glBindTexture (GL_TEXTURE_2D, self->texture);
+
   if (self->pixdata != NULL) {
     *texture_width = retro_pixdata_get_width (self->pixdata);
     *texture_height = retro_pixdata_get_height (self->pixdata);
@@ -287,6 +292,8 @@ retro_gl_display_realize (RetroGLDisplay *self)
 
   if (self->glsl_filter[filter] != NULL)
     retro_glsl_filter_use_program (self->glsl_filter[filter]);
+
+  glClearColor (0, 0, 0, 1);
 }
 
 static void
@@ -313,7 +320,7 @@ retro_gl_display_render (RetroGLDisplay *self)
 
   g_return_val_if_fail (RETRO_IS_GL_DISPLAY (self), FALSE);
 
-  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+  glClear (GL_COLOR_BUFFER_BIT);
 
   filter = self->filter >= RETRO_VIDEO_FILTER_COUNT ?
     RETRO_VIDEO_FILTER_SMOOTH :
