@@ -17,11 +17,14 @@
 #include "retro-core-view.h"
 #include "retro-gtk-version.h"
 #include "retro-input.h"
+#include "retro-key-joypad-mapping.h"
 #include "retro-log.h"
 #include "retro-main-loop.h"
 #include "retro-memory-type.h"
 #include "retro-module-iterator.h"
 #include "retro-module-query.h"
+#include "retro-option.h"
+#include "retro-option-iterator.h"
 #include "retro-pixdata.h"
 #include "retro-video-filter.h"
 
